/*
 * MikroSDK - MikroE Software Development Kit
 * Copyright (c) 2019, MikroElektronika - www.mikroe.com
 * All rights reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*!
 * \file
 *
 */

#include "usbuart.h"

// ------------------------------------------------ PUBLIC FUNCTION DEFINITIONS

void usbuart_cfg_setup ( usbuart_cfg_t *cfg )
{
    // Communication gpio pins

    cfg->rx_pin = HAL_PIN_NC;
    cfg->tx_pin = HAL_PIN_NC;

    // Additional gpio pins

    cfg->cts = HAL_PIN_NC;
    cfg->slp = HAL_PIN_NC;
    cfg->pwr = HAL_PIN_NC;
    cfg->rts = HAL_PIN_NC;

    cfg->baud_rate  = 9600;
    cfg->data_bit   = UART_DEFAULT_DATA_BITS;
    cfg->parity_bit = UART_DEFAULT_PARITY_BIT;
    cfg->stop_bit   = UART_DEFAULT_STOP_BIT;
}

USBUART_RETVAL usbuart_init ( usbuart_t *ctx, usbuart_cfg_t *cfg )
{
    uart_config_t uart_cfg;

    uart_configure_default( &uart_cfg );
    uart_cfg.pins.tx_pin = cfg->tx_pin;
    uart_cfg.pins.rx_pin = cfg->rx_pin;
    uart_cfg.baud        = cfg->baud_rate;
    uart_cfg.data_bits   = cfg->data_bit;
    uart_cfg.parity      = cfg->parity_bit;
    uart_cfg.stop        = cfg->stop_bit;

    if ( uart_open( &ctx->uart, &uart_cfg ) != UART_SUCCESS )
    {
        return USBUART_INIT_ERROR;
    }

    // Output pins

     digital_out_init( &ctx->cts, cfg->cts );
     digital_out_init( &ctx->slp, cfg->slp );
     digital_out_init( &ctx->pwr, cfg->pwr );

    // Input pins

    digital_in_init( &ctx->rts, cfg->rts );

    return USBUART_OK;
}

void usbuart_default_cfg ( usbuart_t *ctx )
{
    // Click default configuration

    usbuart_pwr_ctrl( ctx, USBUART_POWER_ON );
    usbuart_set_cts ( ctx, USBUART_CTS_NO_ACTIVE );
    usbuart_set_mode ( ctx, USBUART_MODE_NORMAL );
}

void usbuart_generic_multi_write ( usbuart_t *ctx, usbuart_data_t *data_buf, uart_length_t len )
{
    uart_block_t block;

    block.buffer = data_buf;
    block.length = len;
    uart_write_block( &ctx->uart, &block);
}

void usbuart_generic_multi_read ( usbuart_t *ctx, usbuart_data_t *data_buf, uart_length_t len )
{
    uart_block_t block;

    block.buffer = data_buf;
    block.length = len;

    uart_read_block( &ctx->uart, &block);
}

usbuart_data_t usbuart_generic_single_read ( usbuart_t *ctx )
{
    usbuart_data_t rx_data;

    rx_data = uart_read( &ctx->uart );

    return rx_data;
}

void usbuart_generic_single_write ( usbuart_t *ctx, usbuart_data_t tx_data )
{
    uart_write( &ctx->uart, tx_data );
}

void usbuart_pwr_ctrl ( usbuart_t *ctx, uint8_t state )
{
    if ( state == 1 )
    {
        digital_out_high( &ctx->pwr );
    }
    else
    {
        digital_out_low( &ctx->pwr );
    }
}

void usbuart_set_cts ( usbuart_t *ctx, uint8_t state )
{
    if ( state == 1 )
    {
        digital_out_high( &ctx->cts );
    }
    else
    {
        digital_out_low( &ctx->cts );
    }
}

void usbuart_set_mode ( usbuart_t *ctx, uint8_t mode )
{
    if ( mode == 1 )
    {
        digital_out_high( &ctx->slp );
    }
    else
    {
        digital_out_low( &ctx->slp );
    }
}

uint8_t usbuart_get_rts ( usbuart_t *ctx )
{
   return  digital_in_read( &ctx->rts );
}

// ------------------------------------------------------------------------- END

